CON

  _CLKMODE            = XTAL1 + PLL16X
  _XINFREQ            = 5_000_000

  'Clock Frequency Equation
  CLK_FREQ            = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ

  MULDIV_DIVISOR   = 1_000_000
  X_STEPS_PER_METER   = 55_555
  X_MULDIV_QN         = X_STEPS_PER_METER / MULDIV_DIVISOR
  X_MULDIV_RN         = X_STEPS_PER_METER // MULDIV_DIVISOR

  TX                  = 30
  RX                  = 31
  COMM_MODE           = 0
  COMM_BAUD_RATE      = 115_200

OBJ

  Terminal:     "FullDuplexSerial"
  
PUB Main | Mantissa, Quotient

  'The ProcessCoordinate method(not shown or included) converts a floating point
  'coordinate into an integer, and the resulting integer is then used to determine
  'the actual number of required steps to make a targeted move.  With an assumed
  'coordinate of 77.77(mm) being passed to the ProcessCoordinate method, a value of 77770
  'would be returned and stored in the local variable Mantissa of the Parse method
  '(not shown or included).  So a value of 77770 is now being assigned to the local variable
  'Mantissa of this method.  Additionally, a starting point of zero(0) is being assumed for
  'testing purposes.  Further calculations would be required if the starting point was not zero.
  Mantissa := 77_770  

  Terminal.Start(RX, TX, COMM_MODE, COMM_BAUD_RATE)
  WAITCNT(CNT + (1 * CLKFREQ))

  Terminal.Tx(13)
  Terminal.Tx(13)
  Terminal.Str(STRING("Distance to target location is: 77.77mm"))
  Terminal.Tx(13)
  Terminal.Tx(13)
  Terminal.Str(STRING("Quotient := uM_DistanceToStepConversion(Mantissa, X_MULDIV_QN, X_MULDIV_RN, MULDIV_DIVISOR)"))
  Terminal.Tx(13)
  Terminal.Tx(13)
  Terminal.Str(STRING("Which is equivalent to:"))
  Terminal.Tx(13)
  Terminal.Tx(13)
  Terminal.Str(STRING("Quotient := uM_DistanceToStepConversion("))
  Terminal.Dec(Mantissa)
  Terminal.Str(STRING(", "))
  Terminal.Dec(X_MULDIV_QN)
  Terminal.Str(STRING(", "))
  Terminal.Dec(X_MULDIV_RN)
  Terminal.Str(STRING(", "))
  Terminal.Dec(MULDIV_DIVISOR)
  Terminal.Str(STRING(")"))
  Terminal.Tx(13)
  Terminal.Tx(13)
  Terminal.Str(STRING("With a return value of:"))
  Terminal.Str(STRING("Quotient := "))

  Quotient := uM_DistanceToStepConversion(Mantissa, X_MULDIV_QN, X_MULDIV_RN, MULDIV_DIVISOR)

  Terminal.Dec(Quotient)
  Terminal.Str(STRING(", which equals the number of steps required to make the targeted move."))

  IF(Quotient == 0)
  
    Terminal.Tx(13)
    Terminal.Tx(13)
    Terminal.Str(STRING("Zero is not an acceptable result :), especially since a move of 77.77mm must be made."))
    Terminal.Tx(13)
    Terminal.Str(STRING("So there must still be a problem within the MulDivQR method,"))
    Terminal.Tx(13)
    Terminal.Str(STRING("and the result should be 4321 steps to move 77.77mm."))

  ELSEIF(Quotient == 4321)

    Terminal.Tx(13)
    Terminal.Tx(13)
    Terminal.Str(STRING("You have successfully obtained the correct answer of 4321 steps!!!!  YIPPEEE"))

PUB uM_DistanceToStepConversion(Distance, MulDivQn, MulDivRn, MulDivDivisor) | Quotient

  Quotient := MulDivQR(Distance, MulDivQn, MulDivRn, MulDivDivisor)

  RETURN Quotient

PUB MulDivQR(Multiplicand, Qn, Rn, Divisor) |  Quotient, Remainder, Negative

  {
  \param Multiplicand
  \param Qn ( = multiplier / Divisor )
  \param Rn ( = multiplier % Divisor )
  \param Divisor
  \retuRn rounded result of Multiplicand * multiplier / Divisor
  }

  Quotient := 0
  Remainder := 0
  Negative := 0

  IF(Multiplicand < 0)
  
    Negative := 1
    Multiplicand := -Multiplicand


  REPEAT
  
    IF(Multiplicand & 1)
     
      Quotient += Qn
      Remainder += Rn
      
        IF(Remainder >= Divisor)
        
          Quotient++
          Remainder -= Divisor
      
    Multiplicand  >>= 1
    Qn <<= 1
    Rn <<= 1
    
    IF Rn >= Divisor
    
      Qn++ 
      Rn -= Divisor

  WHILE Multiplicand  

  'Rounding
  IF(Remainder > (Divisor / 2))
  
    Quotient++

  'Remainder is valid, but not returned
  IF(Negative == 1)

    RETURN -Quotient

  ELSE

    RETURN Quotient